// NVIC driver for nRF52840 | gat 11/04/23
#include "nrf52840.h"
#include "NVIC.h"

#define SET_IRQ(irq,prio) \
	NVIC_SetPriority(irq, prio); \
	NVIC_ClearPendingIRQ(irq); \
	NVIC_EnableIRQ(irq);

static IRQn_Type GetVector(IRQ_Type ir)
{
	switch (ir)
	{
	default:
	case IRQ_Serial0:
		return SPIM0_SPIS0_TWIM0_TWIS0_SPI0_TWI0_IRQn;
	case IRQ_Serial1:
		return SPIM1_SPIS1_TWIM1_TWIS1_SPI1_TWI1_IRQn;
	case IRQ_Timer0:
		return TIMER0_IRQn;
	case IRQ_Timer1:
		return TIMER1_IRQn;
	case IRQ_SPI2:
		return SPIM2_SPIS2_SPI2_IRQn;
	case IRQ_GPIOTE:
		return GPIOTE_IRQn;
	case IRQ_ADC:
		return SAADC_IRQn;
	case IRQ_PWM:
		return PWM0_IRQn;
	case IRQ_Radio:
		return RADIO_IRQn;
	case IRQ_RTC:
		return RTC0_IRQn;
	}
}


void IRQ_SET(IRQ_Type irq, unsigned short priority)
{
	SET_IRQ(GetVector(irq), priority);
}

void IRQ_CLEAR_PENDING(IRQ_Type irq)
{
	NVIC_ClearPendingIRQ(GetVector(irq));
}
