// PWM driver for nRF52840 | gat 11/04/23
#include "PWM.h"
#include "nrf52840.h"
#include "nrf52840_bitfields.h"
#include "PWM.h"
#include "NVIC.h"
#include "IRQ_Priority.h"

#define FLAG_CLEAR(reg) \
	reg->EVENTS_LOOPSDONE = 0; \
	reg->EVENTS_SEQEND[0] = 0; \
	reg->EVENTS_SEQEND[1] = 0; \
	reg->EVENTS_STOPPED = 0;

#define SHORT_SIZE(array)  (sizeof(array) / sizeof(uint16_t))
static uint16_t PWM_High_Level_Periods[] = { 50, 0, 0, 0 };

static void PWM_Set_Sequence(volatile PWM_SEQ_Type* reg)
{
	reg->PTR = (unsigned int)PWM_High_Level_Periods;
	reg->CNT = SHORT_SIZE(PWM_High_Level_Periods);
	reg->ENDDELAY = 0;
	reg->REFRESH = 0;
}

static NRF_PWM_Type* GetReg(PWM_Channel ch)
{
	switch(ch)
	{
		default:
		case PWM0:
		return NRF_PWM0;
		case PWM1:
		return NRF_PWM1;
		case PWM2:
		return NRF_PWM2;
		case PWM3:
		return NRF_PWM3;
	}
}

void PWM_Setup(PWM_Config cfg)
{
	NRF_PWM_Type* reg = GetReg(cfg.Channel);
	reg->PSEL.OUT[0] = cfg.PWM_Pin + (cfg.PWM_Port<<5U); // We need to select the pins first
	reg->ENABLE = PWM_ENABLE_ENABLE_Msk;
	reg->PRESCALER = PWM_PRESCALER_PRESCALER_DIV_32 ; /*!< Divide by 32 (500 kHz) */
	reg->MODE = 0U; // Classic use
	reg->DECODER = PWM_DECODER_LOAD_Common; // PWM_DECODER_LOAD_Common ou PWM_DECODER_LOAD_Individual
	reg->SHORTS = 0U;
	reg->INTEN = 0U; // No interrupt
	FLAG_CLEAR(reg);
	IRQ_SET(IRQ_PWM, PWM_PRIO);
}

void PWM_Start(PWM_Channel chann)
{
	NRF_PWM_Type* reg = GetReg(chann);
	reg->TASKS_SEQSTART[1] = 1;
}

void PWM_Stop(PWM_Channel chann)
{
	NRF_PWM_Type* reg = GetReg(chann);
	reg->TASKS_STOP = 1;
}

void PWM_Update(PWM_Channel chann, int frequency, int percentage)
{
	NRF_PWM_Type* reg = GetReg(chann);
	//percentage = 100 - percentage; // duty cycle is inverted but inverted again in the aeroneb board
	PWM_Set_Sequence(&reg->SEQ[0]);
	PWM_Set_Sequence(&reg->SEQ[1]);
	reg->LOOP = 1;
	reg->SHORTS = PWM_SHORTS_LOOPSDONE_SEQSTART1_Msk;
	uint32_t regval = 500000 / frequency;
	reg->COUNTERTOP = regval;
	for(int i = 0; i < SHORT_SIZE(PWM_High_Level_Periods); i++) PWM_High_Level_Periods[i] = regval / 100 * percentage; // 0 = 0 and 100 = 25000
	reg->EVENTS_STOPPED = 0;
}

void PWM0_IRQHandler()
{
	NRF_PWM_Type* reg = GetReg(PWM0);
	FLAG_CLEAR(reg);
}

